import EventEmitter from '../lib/EventEmitter'
import { apiMark } from '../qa-test-runner'
import { API_QA_STATUS } from '../constants'

export function formatTestError(testError) {
  var err = testError || {}
  if (err instanceof Error) {
    err = errorJSON(err)
  }
  return cleanCycles(err)
}

/**
 * Transform an Error object into a JSON object.
 *
 * @private
 * @param {Error} err
 * @return {Object}
 */
function errorJSON(err) {
  var res = {}
  Object.getOwnPropertyNames(err).forEach(function(key) {
    res[key] = err[key]
  }, err)
  return res
}

function clean(test) {
  return {
    title: test.title,
    fullTitle: test.fullTitle(),
    file: test.file,
    duration: test.duration,
    currentRetry: test.currentRetry(),
    speed: test.speed,
    err: formatTestError(test.err)
  }
}

/**
 * Replaces any circular references inside `obj` with '[object Object]'
 *
 * @private
 * @param {Object} obj
 * @return {Object}
 */
function cleanCycles(obj) {
  var cache = []
  return JSON.parse(
    JSON.stringify(obj, function(key, value) {
      if (typeof value === 'object' && value !== null) {
        if (cache.indexOf(value) !== -1) {
          // Instead of going in a circle, we'll print [object Object]
          return '' + value
        }
        cache.push(value)
      }

      return value
    })
  )
}

function getPathFromTitle(title) {
  const RE = /\{(.*)\}/
  const match = title.match(RE)
  return (match ? match[1] : '').trim()
}

function getApiInfoFromTest(test, status, onReturnBefore) {
  const title = test.title
  const key = getPathFromTitle(title)

  const api = apiMark[key]
  if (api === undefined) {
    throw new Error(`The ${key} key is not exist in apiMark`)
  }

  const wrapper = api.value
  const excutor = wrapper.excutor
  const lastReturn = wrapper.excutorReturns[wrapper.excutorReturns.length - 1]

  const testInfo = {
    sort: wrapper.sort,
    status,
    title: title,
    error: test.err || null,
    returnValue: lastReturn,
    args: excutor.args.length ? excutor.args[excutor.args.length - 1] : [],
    // 使用 专门给用户看的字段
    expectValue: wrapper.expectValues.length
      ? wrapper.expectValues[wrapper.expectValues.length - 1]
      : void 0
  }

  onReturnBefore && onReturnBefore(testInfo)

  return testInfo
}

class TestResultAnalyzer extends EventEmitter {
  recordList = []
  recordCursor = null

  tests = []
  pending = []
  failures = []
  passes = []
  skips = []

  analyze(type, test) {
    switch (type) {
      case 'test-fail':
        this.failures.push(test)
        this.emit(type, getApiInfoFromTest(test, API_QA_STATUS.FAIL))
        break
      case 'test-pass':
        this.passes.push(test)
        this.emit(type, getApiInfoFromTest(test, API_QA_STATUS.PASS, (testInfo) => {
          const returnValue = testInfo.returnValue
          const hasError = returnValue instanceof Error
          // 我们判断这种状态，是人为导致的
          if (hasError) {
            testInfo.status = API_QA_STATUS.USER_PASS
            testInfo.error = returnValue
          }
        }))
        break
      case 'test-skip':
        this.skips.push(test)
        this.emit(type, getApiInfoFromTest(test, API_QA_STATUS.SKIP))
        break
      case 'test-pending':
        this.pending.push(test)
        this.emit(type, getApiInfoFromTest(test, API_QA_STATUS.PEND))
        break
      case 'test-end':
        this.tests.push(test)
        break
    }
  }

  finish(runner) {
    this.record(runner)
    this.emit('finish', this)
  }

  resetQueue() {
    this.tests = []
    this.pending = []
    this.failures = []
    this.passes = []
    this.skips = []
  }

  getCurrentRecord() {
    if (this.recordCursor === null) {
      throw new Error('recordCursor is not be recorded')
    }

    return this.recordCursor
  }

  record(runner) {
    const { tests, pending, failures, passes, skips } = this

    const perRecord = (this.recordCursor = {
      stats: runner.stats,
      tests: tests.map(clean),
      pending: pending.map(clean),
      failures: failures.map(clean),
      passes: passes.map(clean),
      skips: skips.map(clean)
    })

    this.recordList.push(perRecord)
    this.resetQueue()
  }
}

export default TestResultAnalyzer
