import React, { Component } from 'react'
import QAReporter from '../QA-handle/QAReporter'
import TestResultAnalyzer, {
  formatTestError
} from '../QA-handle/TestResultAnalyzer'
import { injectTestSuite } from '../qa-test-runner'
import { RUN_TIME_STATUS, API_QA_STATUS, SUBMIT_API_URL, IS_DEV_BUILD_TYPE } from '../constants'
import { connect } from 'react-redux'
import polyfillSDK from '../utils/polyfillSDK'
import { toString } from '../utils/shared'
import { UI } from '@hyext/hy-ui'
import logger from '../utils/logger'
import { hostContext } from '../qa-test-runner'
import "./Home/index.hycss";

const { View, Dialog, Input, Tip, Form, Switch } = UI

// rawApiList 改成从redux拿
export const createTestHandleHOC = component => {
  class TestHandle extends Component {
    testResultAnalyzer = null
    $dialog = null
    pendingReportData = null

    state = { runtimeStatus: RUN_TIME_STATUS.NOOP }

    initTestEnv() {
      const { add, update } = this.props
      const testResultAnalyzer = (this.testResultAnalyzer = new TestResultAnalyzer())

      testResultAnalyzer.on('test-fail', update)
      testResultAnalyzer.on('test-pass', update)
      testResultAnalyzer.on('test-skip', update)
      testResultAnalyzer.on('test-pending', add)

      testResultAnalyzer.on('finish', () => {
        UI.Tip.show('测试结束')
        this.setState({ runtimeStatus: RUN_TIME_STATUS.END })
      })
    }

    componentDidMount() {
      this.initTestEnv()
    }

    createMochaInstance(apiQueue) {
      const mocha = new Mocha(hostContext.mocha)

      mocha.reporter(QAReporter, { dataAnalyzer: this.testResultAnalyzer })
      mocha.cleanReferencesAfterRun(false)

      mocha.ui('bdd')
      mocha.suite.emit('pre-require', global, null, mocha) // inject global interface

      injectTestSuite(apiQueue)

      return mocha
    }

    handleTestStart = apiQueue => {
      this.setState({ runtimeStatus: RUN_TIME_STATUS.ING }, () => {
        const mocha = this.createMochaInstance(apiQueue)

        this.props.reset()

        console.log('测试开始')
        mocha.run(failures => {
          console.log('测试完成', failures)
        })
      })
    }

    handleReportStart = data => {
      this.pendingReportData = data
      this.$dialog.open()
    }

    beginReportData = formData => {
      const pendingReportData = this.pendingReportData
      this.reportData({ ...formData, ...pendingReportData })
    }

    reportData = data => {
      const { apiList, reportWeChat, ...qaRecord } = data

      function totalTestCount(tests, status) {
        return tests.filter(test => test.status === status).length
      }

      const apiInfoList = apiList.map(item => {
        const tests = item.tests

        return {
          caseFailNum: totalTestCount(tests, API_QA_STATUS.FAIL),
          caseSuccessNum: totalTestCount(tests, API_QA_STATUS.PASS),
          caseSkipNum: totalTestCount(tests, API_QA_STATUS.SKIP),
          name: item.name,
          path: item.path,
          status: item.status,
          testInfoList: tests.map(test => {
            return {
              status: test.status,
              args: toString(test.args),
              expectValue: toString(test.expectValue),
              returnValue: toString(test.returnValue),
              error: toString(test.error),
              title: test.title
            }
          })
        }
      })

      const postData = {
        qaRecord,
        apiInfoList,
        reportWeChat
      }

      // 上报调用的逻辑以后会通过事件解耦
      logger.log('postData', postData)

      polyfillSDK
        .request({
          url: SUBMIT_API_URL,
          method: 'PUT',
          data: postData,
          dataType: 'json',
          isDirect: IS_DEV_BUILD_TYPE
        })
        .then(res => {
          if (res) {
            UI.Tip.show('上报成功')
          } else {
            UI.Tip.show('上报失败')
          }
        })
    }

    render() {
      const { rawApiList } = this.props
      const ReactComponent = component

      return (
        <>
          <ReactComponent
            {...this.props}
            rawApiList={rawApiList}
            onStartTest={this.handleTestStart}
            onStartReport={this.handleReportStart}
            runtimeStatus={this.state.runtimeStatus}
          ></ReactComponent>
          <ReportDialog
            ref={c => {
              this.$dialog = c
            }}
            onConfirm={this.beginReportData}
          ></ReportDialog>
        </>
      )
    }
  }

  class ReportDialog extends Component {
    state = { comment: '', reportWeChat: false }

    renderDialogBody() {
      const { comment, reportWeChat } = this.state
      return (
        <View className="input-wrapper">
          <Form>
            <Form.Item label="备注">
              <Input
                value={comment}
                placeholder="提交备注(必填)"
                onChange={v => {
                  this.setState({ comment: v })
                }}
              ></Input>
            </Form.Item>
            <Form.Item label="是否上报至企业微信">
              <Switch
                value={reportWeChat}
                placeholder="提交备注(必填)"
                onChange={v => {
                  this.setState({ reportWeChat: v })
                }}
              ></Switch>
            </Form.Item>
          </Form>
        </View>
      )
    }

    open() {
      return this.$dialog.open()
    }

    close() {
      return this.$dialog.close()
    }

    handleConfirm = () => {
      const { onConfirm } = this.props
      const { comment, reportWeChat } = this.state
      onConfirm && onConfirm({ comment, reportWeChat })
    }

    render() {
      return (
        <Dialog
          ref={c => {
            this.$dialog = c
          }}
          title="QA上报"
          body={this.renderDialogBody()}
          confirmCallback={this.handleConfirm}
        ></Dialog>
      )
    }
  }

  const TestHandleConnect = connect(
    store => store.reducer,
    dispatch => {
      return {
        add: apiInfo => {
          dispatch({
            type: 'ADD',
            data: apiInfo
          })
        },
        update: apiInfo => {
          dispatch({
            type: 'UPDATE',
            data: apiInfo
          })
        },
        reset: () => {
          dispatch({
            type: 'RESET',
            data: null
          })
        }
      }
    }
  )(TestHandle)

  return TestHandleConnect
}
